/* ---------------------------------------------------------------------------
 *  This software is in the public domain, furnished "as is", without technical
 *  support, and with no warranty, express or implied, as to its usefulness for
 *  any purpose. For more see LICENSE.txt that has to accompany this file.
 *
 *  ProgressReporter.h
 *
 *  Copyright Aleksey Boyko, Princeton University, 2013.
 *
 *  Description: Simple progress reporter class.
 *
 *  -------------------------------------------------------------------------*/

#ifndef PROGRESSREPORTER_H_
#define PROGRESSREPORTER_H_

#include <stdio.h>

#include "RNBasics/RNBasics.h"

namespace igra {

class ProgressReporter {
public:
    ProgressReporter(const char *message, bool report, FILE *io = stdout);
    virtual ~ProgressReporter();

    void update(RNScalar progress);
    void set_term_status(const char* status);
protected:
    FILE *dev_;
    char* msg_;
    char* term_stat_;
    bool print_;
    RNTime timer_;
};

inline void ProgressReporter::update(RNScalar progress) {
    if (print_)
        fprintf(dev_, "\r\t\t%-40s progress %5.2f%%", (msg_ ? msg_ : ""),
                progress);
}

inline void ProgressReporter::set_term_status(const char* status) {
    if (term_stat_) free(term_stat_);
    if (status) term_stat_ = strdup(status);
}

} // namespace igra

#endif /* PROGRESSREPORTER_H_ */
