/* ---------------------------------------------------------------------------
 *  This software is in the public domain, furnished "as is", without technical
 *  support, and with no warranty, express or implied, as to its usefulness for
 *  any purpose. For more see LICENSE.txt that has to accompany this file.
 *
 *  R3SurfelLabelerCheckpoint.h
 *
 *  Copyright Aleksey Boyko, Princeton University, 2014.
 *
 *  Description: Checkpoint class that stores, reads and writes a snapshot of
 *               the annotation progress.
 *
 *  -------------------------------------------------------------------------*/

#ifndef R3SURFELLABELERCHECKPOINT_H_
#define R3SURFELLABELERCHECKPOINT_H_

#include "R3Surfels/R3Surfels.h"

namespace igra {

// Forward declarations.
class R3SurfelLabeler;

class R3SurfelLabelerCheckpoint {
public:
    virtual ~R3SurfelLabelerCheckpoint();

    // I/O.
    static void PrintCheckpoint(FILE *fp, const R3SurfelLabeler *labeler);
    static R3SurfelLabelerCheckpoint* TryParseCheckPoint(FILE *fp,
            R3SurfelScene *scene);

    // Getters/setters.
    RNScalar get_timestamp() const;
    void set_timestamp(RNScalar timestamp);
    RNScalar get_predictionf() const;
    RNScalar get_confirmed_fraction() const;

protected:
    R3SurfelLabelerCheckpoint();

    // Time since beginning of execution
    RNScalar timestamp;

    int labels_num;
    int gt_obj_num, hl_obj_num, pred_obj_num;
    RNScalar prediction_precision, prediction_recall, prediction_f;
    RNScalar human_precision, human_recall, human_f;
};

// Inline methods.

inline RNScalar R3SurfelLabelerCheckpoint::get_timestamp() const {
    return timestamp;
}

inline void R3SurfelLabelerCheckpoint::set_timestamp(RNScalar timestamp) {
    this->timestamp = timestamp;
}

inline RNScalar R3SurfelLabelerCheckpoint::get_predictionf() const {
    return prediction_f;
}

inline RNScalar R3SurfelLabelerCheckpoint::get_confirmed_fraction() const {
    if (gt_obj_num == 0) return 0;
    return hl_obj_num / (RNScalar) gt_obj_num;
}

} // namespace igra

#endif /* R3SURFELLABELERCHECKPOINT_H_ */
