/* ---------------------------------------------------------------------------
 *  This software is in the public domain, furnished "as is", without technical
 *  support, and with no warranty, express or implied, as to its usefulness for
 *  any purpose. For more see LICENSE.txt that has to accompany this file.
 *
 *  R3SurfelLabelerCommand.h
 *
 *  Copyright Aleksey Boyko, Princeton University, 2014.
 *
 *  Description: A class that stores, reads and writes interface commands.
 *
 *  -------------------------------------------------------------------------*/

#ifndef R3SURFELLABELERCOMMAND_H_
#define R3SURFELLABELERCOMMAND_H_

#include "R3Graphics/R3Graphics.h"
#include "R3Surfels/R3Surfels.h"

namespace igra {

// Command types
enum Commands {
    R3_SURFEL_LABELER_INITIALIZE_COMMAND = 0,  //0
    R3_SURFEL_LABELER_TERMINATE_COMMAND,  //1
    R3_SURFEL_LABELER_SYNC_COMMAND,  //2
    R3_SURFEL_LABELER_ZOOM_COMMAND,  //3
    R3_SURFEL_LABELER_CAPTURE_IMAGE_COMMAND,  //4
    R3_SURFEL_LABELER_SELECT_NONE_COMMAND,  //5
    R3_SURFEL_LABELER_SELECT_PICKED_COMMAND,  //6
    R3_SURFEL_LABELER_SELECT_ENCLOSED_COMMAND,  //7
    R3_SURFEL_LABELER_SELECT_SUGGESTED_COMMAND,  //8
    R3_SURFEL_LABELER_SELECT_ALL_COMMAND,  //9
    R3_SURFEL_LABELER_LABEL_PICKED_COMMAND,  //10
    R3_SURFEL_LABELER_LABEL_SELECTION_COMMAND,  //11
    R3_SURFEL_LABELER_CONFIRM_PICKED_COMMAND,  //12
    R3_SURFEL_LABELER_CONFIRM_SELECTION_COMMAND,  //13
    R3_SURFEL_LABELER_CONFIRM_ALL_COMMAND,  //14
    R3_SURFEL_LABELER_REFUTE_PICKED_COMMAND,  //15
    R3_SURFEL_LABELER_REFUTE_SELECTION_COMMAND,  //16
    R3_SURFEL_LABELER_REFUTE_ALL_COMMAND,  //17
    R3_SURFEL_LABELER_SET_BACKGROUND_VISIBILITY_COMMAND,  //18
    R3_SURFEL_LABELER_SET_LABEL_NAME_VISIBILITY_COMMAND,  //19
    R3_SURFEL_LABELER_SET_SURFEL_COLOR_SCHEME_COMMAND,  //20
    R3_SURFEL_LABELER_SET_LABEL_VISIBILITY_COMMAND,  //21
    R3_SURFEL_LABELER_UNDO_COMMAND,  //22
    R3_SURFEL_LABELER_REDO_COMMAND,  //23
    R3_SURFEL_LABELER_RESET_COMMAND,  //24
    R3_SURFEL_LABELER_BATCH_TRAINING_BEGIN,  //25
    R3_SURFEL_LABELER_PREDICT_ALL_COMMAND,  //26
    R3_SURFEL_LABELER_PAUSE,    //27
    R3_SURFEL_LABELER_MERGE_OBJECTS,  //28
    R3_SURFEL_LABELER_SPLIT_OBJECTS,  //29
    R3_SURFEL_LABELER_ADD_OBJECT,  //30 -> not used/implemented
    R3_SURFEL_LABELER_DELETE_OBJECT,  //31 -> not used/implemented
    R3_SURFEL_LABELER_ADD_LABEL,  //32 -> not used/implemented
    R3_SURFEL_LABELER_DELETE_LABEL,  //33 -> not used/implemented
    R3_SURFEL_LABELER_SUGGEST_INCREASE_SELECTION, //34
    R3_SURFEL_LABELER_SUGGEST_DECREASE_SELECTION, //35
    R3_SURFEL_LABELER_LABEL_TRAINING_MISTAKE_COMMAND,  //36
    R3_SURFEL_LABELER_NUM_COMMANDS  //37
};

// Forward declarations
class R3SurfelLabeler;
class UserSession;

// Command class definition

class R3SurfelLabelerCommand {
public:
    // Constructor
    R3SurfelLabelerCommand(R3SurfelLabeler *labeler, int type, double operand =
            0);
    virtual ~R3SurfelLabelerCommand();

    // IO.
    void Write(FILE *fp) const;
    static R3SurfelLabelerCommand* TryParseCommand(FILE *fp,
            R3SurfelScene *scene);

    // Getters.
    RNScalar get_operand() const;
    int get_type() const;
    RNScalar get_timestamp() const;
    void set_timestamp(RNScalar timestamp);
    const RNArray<R3SurfelLabelAssignment*>& get_inserted_label_assignments() const;
    const RNArray<R3SurfelObject*>& get_removed_object_selections() const;

protected:
    R3SurfelLabelerCommand();

protected:
    // Data.

    // Command type
    int type;

    // Command operand
    RNScalar operand;

    // Labeler state before command was executed
    R3Camera camera;
    R3Point center_point;

    // Time since beginning of execution
    RNScalar timestamp;

    // Object selections
    RNArray<R3SurfelObject *> inserted_object_selections;
    RNArray<R3SurfelObject *> removed_object_selections;

    // Label assignments
    RNArray<R3SurfelLabelAssignment *> inserted_label_assignments;
    RNArray<R3SurfelLabelAssignment *> removed_label_assignments;

    // Inidcator of this command owning assignment pointers.
    bool own_assignments;

    // This is only used by R3SurfelLabeler
    friend class R3SurfelLabeler;
    friend class UserSession;
};

// Inline methods.

inline int R3SurfelLabelerCommand::get_type() const {
    return type;
}

inline RNScalar R3SurfelLabelerCommand::get_operand() const {
    return operand;
}

inline RNScalar R3SurfelLabelerCommand::get_timestamp() const {
    return timestamp;
}

inline void R3SurfelLabelerCommand::set_timestamp(RNScalar timestamp) {
    this->timestamp = timestamp;
}

inline const RNArray<R3SurfelLabelAssignment*>& R3SurfelLabelerCommand::
get_inserted_label_assignments() const {
    return inserted_label_assignments;
}

inline const RNArray<R3SurfelObject*>& R3SurfelLabelerCommand::
get_removed_object_selections() const {
    return removed_object_selections;
}

} // namespace igra

#endif /* R3SURFELLABELERCOMMAND_H_ */
