/* ---------------------------------------------------------------------------
 *  This software is in the public domain, furnished "as is", without technical
 *  support, and with no warranty, express or implied, as to its usefulness for
 *  any purpose. For more see LICENSE.txt that has to accompany this file.
 *
 *  SelectionContainer.h
 *
 *  Copyright Aleksey Boyko, Princeton University, 2013.
 *
 *  Description: Helper class for selecting/drawing nodes.
 *
 *  -------------------------------------------------------------------------*/

#include "SelectionContainer.h"

#include <limits>

namespace igra {

const RNScalar kMaxScalar = numeric_limits<int>::max();

SelectionContainer::SelectionContainer() {
    Reset();
}

SelectionContainer::~SelectionContainer() {
}

void SelectionContainer::Reset(const R2Box &box) {
    closest_dist = kMaxScalar;
    world_selection_box = box;
    closest = NULL;
    current = NULL;
}

void SelectionContainer::try_update_with_current(const R2Point &p) {
    if (!current) return;
    if (!R2Contains(world_selection_box, p)) return;
    RNScalar dist = R2SquaredDistance(p, world_selection_box.Centroid());
    if (dist < closest_dist) {
        closest = current;
        closest_dist = dist;
    }
}

} // namespace igra
