/* ---------------------------------------------------------------------------
 *  This software is in the public domain, furnished "as is", without technical
 *  support, and with no warranty, express or implied, as to its usefulness for
 *  any purpose. For more see LICENSE.txt that has to accompany this file.
 *
 *  SelectionContainer.h
 *
 *  Copyright Aleksey Boyko, Princeton University, 2013.
 *
 *  Description: Helper class for selecting/drawing nodes.
 *
 *  -------------------------------------------------------------------------*/

#ifndef SELECTIONCONTAINER_H_
#define SELECTIONCONTAINER_H_

#include <list>
#include <string>

#include "R2Shapes/R2Shapes.h"
#include "ObjectWrapper.h"
#include "TreeTraits.h"

using namespace std;

namespace igra {

class SelectionContainer {
public:
    // Constructor/destructor.
    SelectionContainer();
    virtual ~SelectionContainer();

    // Mutators. //

    template<class Node>
    void set_current(Node *node);
    template<class Node>
    void set_closest(Node *node);

    void Reset(const R2Box &box = R2null_box);
    void try_update_with_current(const R2Point &p);

    // Accessors. //

    template<class Node>
    bool is_closest(const Node *node) const;

    template<class Node>
    void get_closest(Node **node) const;

    bool something_selected() const;
    R3SurfelObject* object_selected() const;
    R3SurfelObject* current_object() const;

    void FillStats(list<string> &data) const;

protected:
    R2Box world_selection_box;
    RNScalar closest_dist;

    ProtoTreeTraits<ObjectWrapper> *closest;
    ProtoTreeTraits<ObjectWrapper> *current;
};

////////////////////////////////////////////////////////////////////////
// Inline/template methods.
////////////////////////////////////////////////////////////////////////

template<class Node>
void SelectionContainer::set_current(Node *node) {
    current = node;
}

template<class Node>
void SelectionContainer::set_closest(Node *node) {
    closest = node;
}

template<class Node>
bool SelectionContainer::is_closest(const Node *node) const {
    return (node && closest) ? node == static_cast<Node*>(closest) : false;
}

template<class Node>
void SelectionContainer::get_closest(Node **node) const {
    if (!node) return;

    *node = NULL;
    if (something_selected()) {
        *node = static_cast<Node*>(closest);
    }
}

inline R3SurfelObject* SelectionContainer::object_selected() const {
    return (closest && closest->object()) ? closest->object()->object() : NULL;
}

inline bool SelectionContainer::something_selected() const {
    return closest != NULL;
}

inline R3SurfelObject* SelectionContainer::current_object() const {
    return (current && current->object()) ? current->object()->object() : NULL;
}

inline void SelectionContainer::FillStats(list<string> &data) const {
    if (closest) closest->FillStats(data);
}

} // namespace igra

#endif /* SELECTIONCONTAINER_H_ */
