/* ---------------------------------------------------------------------------
 *  This software is in the public domain, furnished "as is", without technical
 *  support, and with no warranty, express or implied, as to its usefulness for
 *  any purpose. For more see LICENSE.txt that has to accompany this file.
 *
 *  UserSession.h
 *
 *  Copyright Aleksey Boyko, Princeton University, 2014.
 *
 *  Description: Helper that streamlines interaction with the heterogeneous
 *               contents of the user log files.
 *
 *  -------------------------------------------------------------------------*/


#ifndef USERSESSION_H_
#define USERSESSION_H_

#include "R3SurfelLabelerCheckpoint.h"
#include "R3SurfelLabelerCommand.h"
#include "RNBasics/RNBasics.h"

namespace igra {

// Forward declarations.
class R3SurfelLabelerCommand;
class R3SurfelLabelerCheckpoint;

class UserSession {
public:

    // Constructor/ destructor.
    UserSession();
    ~UserSession();

    // Mutators.
    void clear(bool deallocate = true);
    void trim_to_last_checkpoint();

    // Forward iterator to facilitate traversal of the session log.
    class Iterator {
    public:
        // Constructor destructor.
        Iterator();
        Iterator(const UserSession *session);
        Iterator(const Iterator &it);

        // static methods.
        static Iterator begin(const UserSession *session);
        static Iterator end(const UserSession *session);
        static Iterator back(const UserSession *session);

        // Accessors.
        R3SurfelLabelerCommand* command() const;
        R3SurfelLabelerCheckpoint* checkpoint() const;
        RNScalar timestamp() const;

        // Mutators.
        void adjust_time(RNScalar new_timestamp);

        // Operators.
        Iterator& operator++();
        Iterator& operator=(const Iterator &it);
        bool operator!=(const Iterator &it) const;
        bool operator==(const Iterator &it) const;

    protected:
        UserSession *session_;

        int cmd_index_;
        int checkpoint_index_;
        bool use_command_;
    };

    void check_validity();

    bool empty() const;

    void push_back(const Iterator &it);
    void push_back(R3SurfelLabelerCommand *cmd);
    void push_back(R3SurfelLabelerCheckpoint *checkpoint);
    void pop_back();

    Iterator begin() const;
    Iterator end() const;
    Iterator front() const;
    Iterator back() const;

protected:
    RNArray<R3SurfelLabelerCommand*> commands;
    RNArray<R3SurfelLabelerCheckpoint*> checkpoints;
};


// Inline methods.

inline bool UserSession::
empty() const {
    return !(commands.NEntries() + checkpoints.NEntries());
}


inline UserSession::Iterator UserSession::
begin() const {
    return Iterator::begin(this);
}


inline UserSession::Iterator UserSession::
end() const {
    return Iterator::end(this);
}


inline UserSession::Iterator UserSession::
front() const {
    return begin();
}


inline UserSession::Iterator UserSession::
back() const {
    return Iterator::back(this);
}

} // namespace igra

#endif /* USERSESSION_H_ */
