/* Source file for the R3 surfel feature class */



////////////////////////////////////////////////////////////////////////
// INCLUDE FILES
////////////////////////////////////////////////////////////////////////

#include "R3Surfels/R3Surfels.h"



////////////////////////////////////////////////////////////////////////
// CONSTRUCTORS/DESTRUCTORS
////////////////////////////////////////////////////////////////////////

R3SurfelFeature::
R3SurfelFeature(const char *name, RNScalar minimum, RNScalar maximum, RNScalar weight, int id)
  : name((name) ? strdup(name) : NULL),
    range(minimum, maximum),
    weight(weight),
    id(id)
{
  // Set ID
  static int next_id = 0;
  if (id == -1) id = next_id++;
  else if (id >= next_id) next_id = id + 1;
}



R3SurfelFeature::
~R3SurfelFeature(void)
{
  // Delete name
  if (name) free(name);
}



////////////////////////////////////////////////////////////////////////
// MANIPULATION FUNCTIONS
////////////////////////////////////////////////////////////////////////

void R3SurfelFeature::
SetName(const char *name) 
{
  // Copy name
  if (this->name) free(this->name);
  if (name) this->name = strdup(name);
}



void R3SurfelFeature::
SetRange(const RNInterval& range) 
{
  // Set range
  this->range = range;
}



void R3SurfelFeature::
SetRange(RNScalar minimum, RNScalar maximum) 
{
  // Set range
  this->range.Reset(minimum, maximum);
}



void R3SurfelFeature::
SetMinimum(RNScalar minimum) 
{
  // Set minimum
  range.SetMin(minimum);
}



void R3SurfelFeature::
SetMaximum(RNScalar maximum) 
{
  // Set maximum
  range.SetMax(maximum);
}



void R3SurfelFeature::
SetWeight(RNScalar weight) 
{
  // Set weight
  this->weight = weight;
}



////////////////////////////////////////////////////////////////////////
// DISPLAY FUNCTIONS
////////////////////////////////////////////////////////////////////////

void R3SurfelFeature::
Print(FILE *fp, const char *prefix, const char *suffix) const
{
  // Check fp
  if (!fp) fp = stdout;

  // Print feature name
  if (prefix) fprintf(fp, "%s", prefix);
  fprintf(fp, "%s ", Name());
  if (suffix) fprintf(fp, "%s", suffix);
  fprintf(fp, "\n");
}


