/* Include file for the R3 surfel feature class */



/* Class definition */

class R3SurfelFeature {
public:
  //////////////////////////////////////////////
  //// CONSTRUCTOR AND DESTRUCTOR FUNCTIONS ////
  //////////////////////////////////////////////

  // Constructor functions
  R3SurfelFeature(const char *name = NULL, 
    RNScalar minimum = -FLT_MAX, RNScalar maximum = FLT_MAX, 
    RNScalar weight = 1, int id = -1);
  R3SurfelFeature(const R3SurfelFeature& surfel);

  // Destructor functions
  virtual ~R3SurfelFeature(void);


  ////////////////////////////
  //// PROPERTY FUNCTIONS ////
  ////////////////////////////

  // Name properties
  const char *Name(void) const;

  // Range properties
  const RNInterval& Range(void) const;
  RNScalar Minimum(void) const;
  RNScalar Maximum(void) const;

  // Weight property
  RNScalar Weight(void) const;

  // ID property
  int ID(void) const;


  ///////////////////////////////
  //// MANIPULTION FUNCTIONS ////
  ///////////////////////////////

  // Name manipulation
  void SetName(const char *name);

  // Range manipulation
  virtual void SetRange(const RNInterval& range);
  virtual void SetRange(RNScalar minimum, RNScalar maximum);
  virtual void SetMinimum(RNScalar minimum);
  virtual void SetMaximum(RNScalar maximum);

  // Weight manipulation
  virtual void SetWeight(RNScalar weight);


  ///////////////////////////
  //// DISPLAY FUNCTIONS ////
  ///////////////////////////

  // Print function
  virtual void Print(FILE *fp = NULL, const char *prefix = NULL, const char *suffix = NULL) const;


  ////////////////////////////////////////////////////////////////////////
  // INTERNAL STUFF BELOW HERE
  ////////////////////////////////////////////////////////////////////////

private:
  char *name;
  RNInterval range;
  RNScalar weight;
  int id;
};



////////////////////////////////////////////////////////////////////////
// INLINE FUNCTION DEFINITIONS
////////////////////////////////////////////////////////////////////////

inline const char *R3SurfelFeature::
Name(void) const
{
  // Return name
  return name;
}



inline const RNInterval& R3SurfelFeature::
Range(void) const
{
  // Return range
  return range;
}



inline RNScalar R3SurfelFeature::
Minimum(void) const
{
  // Return minimum
  return range.Min();
}



inline RNScalar R3SurfelFeature::
Maximum(void) const
{
  // Return maximum
  return range.Max();
}



inline RNScalar R3SurfelFeature::
Weight(void) const
{
  // Return weight
  return weight;
}



inline int R3SurfelFeature::
ID(void) const
{
  // Return ID
  return id;
}



