/* Include file for the R3 surfel feature space class */



////////////////////////////////////////////////////////////////////////
// CLASS DEFINITION
////////////////////////////////////////////////////////////////////////

class R3SurfelFeatureSpace {
public:
  //////////////////////////////////////////
  //// CONSTRUCTOR/DESTRUCTOR FUNCTIONS ////
  //////////////////////////////////////////

  // Constructor functions
  R3SurfelFeatureSpace(void);
  R3SurfelFeatureSpace(const R3SurfelFeatureSpace& space);

  // Destructor function
  virtual ~R3SurfelFeatureSpace(void);


  //////////////////////////
  //// ACCESS FUNCTIONS ////
  //////////////////////////

  // Label access functions
  int NLabels(void) const;
  R3SurfelLabel *Label(int label_index) const;

  // Feature access functions
  int NFeatures(void) const;
  R3SurfelFeature *Feature(int feature_index) const;

  // Object access functions
  int NObjects(void) const;
  R3SurfelObject *Object(int object_index) const;

  // Feature vector access functions
  const R3SurfelFeatureVector *FeatureVector(int object_index) const;
  RNScalar FeatureValue(int object_index, int feature_index) const;


  /////////////////////////////////////////
  //// MANIPULATION FUNCTIONS ////
  /////////////////////////////////////////

  virtual void InsertLabels(const R3SurfelLabelSet& labelset);
  virtual void RemoveLabels(const R3SurfelLabelSet& labelset);

  virtual void InsertLabel(R3SurfelLabel *label);
  virtual void RemoveLabel(R3SurfelLabel *label);

  virtual void InsertFeatures(const R3SurfelFeatureSet& featureset);
  virtual void RemoveFeatures(const R3SurfelFeatureSet& featureset);

  virtual void InsertFeature(R3SurfelFeature *feature);
  virtual void RemoveFeature(R3SurfelFeature *feature);

  virtual void InsertObjects(const R3SurfelObjectSet& objectset);
  virtual void RemoveObjects(const R3SurfelObjectSet& objectset);

  virtual void InsertObject(R3SurfelObject *object);
  virtual void RemoveObject(R3SurfelObject *object);

  virtual void Empty(void);

Use scene labels
Use scene features
Use scene objects


  ///////////////////////////
  //// DISPLAY FUNCTIONS ////
  ///////////////////////////

  // Print function
  virtual void Print(FILE *fp = NULL, const char *prefix = NULL, const char *suffix = NULL) const;


  ////////////////////////////////////////////////////////////////////////
  // INTERNAL STUFF BELOW HERE
  ////////////////////////////////////////////////////////////////////////

private:
  R3SurfelLabelSet labelset;
  R3SurfelFeatureSet featureset;
  R3SurfelObjectSet objectset;
};



////////////////////////////////////////////////////////////////////////
// INLINE FUNCTION DEFINITIONS
////////////////////////////////////////////////////////////////////////

inline int R3SurfelFeatureSpace::
NFeatures(void) const
{
  // Return number of features
  return features.NEntries();
}



inline R3SurfelFeature *R3SurfelFeatureSpace::
Feature(int k) const
{
  // Return kth feature
  return features.Kth(k);
}



inline R3SurfelFeature *R3SurfelFeatureSpace::
operator[](int k) const
{
  // Return kth feature
  return Feature(k);
}





